#ifndef _RWBLOCK_H
#define _RWBLOCK_H
#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */
/*
 * rwblock.h
 * Blocking read/write utility.
 *
 * Author: Phil Burk, http://www.softsynth.com
 *
 * This program uses the PortAudio Portable Audio Library.
 * For more information see: http://www.audiomulch.com/portaudio/
 * Copyright (c) 1999-2000 Ross Bencina and Phil Burk
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files
 * (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * Any person wishing to distribute modifications to the Software is
 * requested to send the modifications to the original developer so that
 * they can be incorporated into the canonical version.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR
 * ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "portaudio.h"
#include "ringbuffer.h"
#include <string.h>
typedef struct {
	RingBuffer   inFIFO;
	RingBuffer   outFIFO;
	PortAudioStream *stream;
	int          bytesPerFrame;
	int          samplesPerFrame;
}RWBL;
/* Values for flags for OpenAudioStream(). */
#define RWBL_READ     (1<<0)
#define RWBL_WRITE    (1<<1)
#define RWBL_READ_WRITE    (RWBL_READ|RWBL_WRITE)
#define RWBL_MONO     (1<<2)
#define RWBL_STEREO   (1<<3)
/************************************************************
 * Write data to ring buffer.
 * Will not return until all the data has been written.
 */
int WriteAudioStream( RWBL *rwbl, void *data, int numFrames );
/************************************************************
 * Read data from ring buffer.
 * Will not return until all the data has been read.
 */
int ReadAudioStream( RWBL *rwbl, void *data, int numFrames );
/************************************************************
 * Opens a PortAudio stream with default characteristics.
 * Allocates RWBL structure.
 *
 * Note that numFrames will be increased internally to try to prevent
 * glitching. So pass zero for minimal latency.
 *
 * flags parameter can be an ORed combination of:
 *    RWBL_INPUT, RWBL_OUTPUT, and either RWBL_MONO or RWBL_STEREO
 */
PaError OpenAudioStream( RWBL **parwPtr, double sampleRate,
				  PaSampleFormat format, int flags );
PaError CloseAudioStream( RWBL *rwbl );
#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* _RWBLOCK_H */
