#ifndef PA_HOST_H
#define PA_HOST_H

/*
 * Host dependant internal API for PortAudio
 *
 * Author: Phil Burk  <philburk@softsynth.com>
 *
 * PortAudio Portable Real-Time Audio Library
 * Latest Version at: http://www.softsynth.com/portaudio/
 * DirectSound and Macintosh Implementation
 * Copyright (c) 1999-2000 Phil Burk
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files
 * (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * Any person wishing to distribute modifications to the Software is
 * requested to send the modifications to the original developer so that
 * they can be incorporated into the canonical version.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR
 * ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <memory.h>
#include <math.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#ifndef SUPPORT_AUDIO_CAPTURE
#define SUPPORT_AUDIO_CAPTURE  (1)
#endif

/************************************************************************************/
/****************** Structures ******************************************************/
/************************************************************************************/
 
typedef struct internalPortAudioStream
{
	void                     *past_DeviceData;
	PortAudioCallback        *past_Callback;
	void                     *past_UserData;
	PaTimestamp               past_FrameCount;
	unsigned long             past_Flags;
	unsigned int              past_NumCallbacks;
	int                       past_IsActive;
	unsigned int              past_FramesPerUserBuffer;
	unsigned int              past_NumberOfBuffers;
	double                    past_SampleRate;           /* Closest supported sample rate. */
	int                       past_NumInputChannels;
	int                       past_NumOutputChannels;
	PaDeviceID                past_InputDeviceID;
	PaDeviceID                past_OutputDeviceID;
	PaSampleFormat            past_InputSampleFormat;
	PaSampleFormat            past_OutputSampleFormat;
	int                       past_BitsPerInputSample;
	int                       past_BitsPerOutputSample;
	void                     *past_InputBuffer;   /* Buffers for user to fill. */
	long                      past_InputBufferSize;
	void                     *past_OutputBuffer;
	long                      past_OutputBufferSize;
	unsigned int              past_BytesPerNativeBuffer;    /* native buffer size in bytes - FIXME - is this used? */
/* For measuring CPU utilization. */
	double                    past_AverageInsideCount;
	double                    past_AverageTotalCount;
	double                    past_Usage;
	int                       past_IfLastExitValid;
} internalPortAudioStream;

#define PA_MAX_DEVICE_INFO   (32)

/************************************************************************************/
/****************** Prototypes ******************************************************/
/************************************************************************************/

PaError PaHost_Init( void );
PaError PaHost_Term( void );

PaError PaHost_OpenStream( internalPortAudioStream   *past );
PaError PaHost_CloseStream( internalPortAudioStream   *past );

PaError PaHost_StartOutput( internalPortAudioStream   *past );
PaError PaHost_StopOutput( internalPortAudioStream   *past );
PaError PaHost_StartInput( internalPortAudioStream   *past );
PaError PaHost_StopInput( internalPortAudioStream   *past );
PaError PaHost_StartEngine( internalPortAudioStream   *past );
PaError PaHost_StopEngine( internalPortAudioStream   *past );

PaError PaHost_StreamActive( internalPortAudioStream   *past );

long Pa_CallConvertInt16( internalPortAudioStream   *past, 
	short *nativeInputBuffer,
	short *nativeOutputBuffer );

void   *PaHost_AllocateFastMemory( long numBytes );
void    PaHost_FreeFastMemory( void *addr, long numBytes );

PaError PaHost_ValidateSampleRate( PaDeviceID id, double requestedFrameRate,
		double *closestFrameRatePtr );
int PaHost_FindClosestTableEntry( double allowableError,  const double *rateTable,
	int numRates, double frameRate );

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* PA_HOST_H */
